* This sample do file implements the estimator of Chetverikov, Larsen, and Palmer (2015)
* Christopher Palmer, cjpalmer@berkeley.edu, July 28, 2015.

set more off
ssc install parmest //install package for graphing coefficients
local quantile_grid = "0.05(.05)0.95"
tempfile coefficients //for saving results

* for testing purposes, load a sample dataset. replace with yours.
sysuse nlsw88, clear
local quantile_grid= "0.3(0.1)0.7" // the sample dataset is small, so need a coarser grid of quantiles
drop if industry==2 | mi(industry)

******** replace these variable definitions with your own, these just work for sample dataset
local yvar = "wage" //dependent variable
local zvars = "age married" //individual-level variables
local xvars = "" //exogenous grouped variables
local endog = "xvar" // endogenous grouped variable
local wvars = "wvar" //instrument
local group_id_vars = "industry" // identifies group

* determine and format group variables
egen group_id=group(`group_id_vars')
qui sum group_id
local group_id_max=r(max)

********* make up some grouped variables since there weren't any in sample dataset
********* this part unnecessary for real data.
egen tag=tag(group_id)
gen xvar1=uniform() if tag
gen xvar2=xvar1*3+uniform() if tag
bys group_id: egen xvar=mean(xvar1)
by group_id: egen wvar=mean(xvar2)
*********

local micro_covs = 1 //change this to 0 if no micro-covariates

* If there are micro-level covariates, first determine conditional quantiles
if `micro_covs'==1 {

	foreach tau of numlist `quantile_grid' {
		local Tau=100*`tau'
		qui gen conditional`Tau'=.
		local collapse_varlist = "`collapse_varlist' conditional`Tau'"

		forvalues group=1/`group_id_max' {

			qui qreg `yvar' `zvars' if group_id==`group', q(`tau')
			qui replace conditional`Tau'=_b[_cons] if group_id==`group'
		}
	}
}

* If there aren't micro covariates, simply calculate conditional quantiles within each group
else {
	
	foreach tau of numlist `quantile_grid' {
		local Tau=`tau'*100
		gen conditional`Tau'=`yvar'
		local collapse_varlist = "`collapse_varlist' (p`Tau') conditional`Tau'"
	}
}

collapse `collapse_varlist' `xvars' `endog' `wvars', by(group_id)


* now run 2SLS on conditional quantiles
foreach tau of numlist `quantile_grid' {
	local Tau=100*`tau'
	ivreg conditional`Tau' `xvars' (`endog'=`wvars'), robust
	preserve
	* this converts dataset into a matrix of coefficients
	parmest, fast
	gen quantile=`tau'
	capture save `coefficients'
	if _rc!=0 append using `coefficients'
	save `coefficients', replace
	restore
	}

	
* now graph coefficients and confidence intervals
use `coefficients', clear
qui keep if parm=="`endog'"
sort quantile
list quantile estimate stderr t p min max
twoway (connected estimate quantile) (rline min95 max95 quantile, lpattern(dash)), legend(label(1 Coefficient) label(2 "95% Confidence Interval"))

* end of do file
